@@ -51,6 +51,9 @@ static bool create_devices_for_terminal_and_renderer_type (ply_device_manager_t
                                                            const char           *device_path,
                                                            ply_terminal_t       *terminal,
                                                            ply_renderer_type_t   renderer_type);
+static void create_pixel_displays_for_renderer (ply_device_manager_t *manager,
+                                                ply_renderer_t       *renderer);
+
 struct _ply_device_manager
 {
         ply_device_manager_flags_t flags;
@@ -81,6 +84,8 @@ struct _ply_device_manager
 
         uint32_t                    paused : 1;
         uint32_t                    device_timeout_elapsed : 1;
+        uint32_t                    found_drm_device : 1;
+        uint32_t                    found_fb_device : 1;
 };
 
 static void
@@ -134,9 +139,38 @@ free_displays_for_renderer (ply_device_manager_t *manager,
         }
 }
 
+static void
+free_keyboards_for_renderer (ply_device_manager_t *manager,
+                            ply_renderer_t       *renderer)
+{
+        ply_list_node_t *node;
+
+        node = ply_list_get_first_node (manager->keyboards);
+        while (node != NULL) {
+                ply_list_node_t *next_node;
+                ply_keyboard_t *keyboard;
+                ply_renderer_t *keyboard_renderer;
+
+                keyboard = ply_list_node_get_data (node);
+                next_node = ply_list_get_next_node (manager->keyboards, node);
+                keyboard_renderer = ply_keyboard_get_renderer (keyboard);
+
+                if (keyboard_renderer == renderer) {
+                        ply_keyboard_free (keyboard);
+                        ply_list_remove_node (manager->keyboards, node);
+                }
+
+                node = next_node;
+        }
+        if (ply_list_get_first_node (manager->keyboards) == NULL) {
+                manager->local_console_managed = false;
+        }
+}
+
 static void
 free_devices_from_device_path (ply_device_manager_t *manager,
-                               const char           *device_path)
+                               const char           *device_path,
+                               bool                  close)
 {
         void *key = NULL;
         void *renderer = NULL;
@@ -150,9 +184,22 @@ free_devices_from_device_path (ply_device_manager_t *manager,
                 return;
 
         free_displays_for_renderer (manager, renderer);
+        free_keyboards_for_renderer (manager, renderer);
 
         ply_hashtable_remove (manager->renderers, (void *) device_path);
         free (key);
+
+        /*
+         * Close is false when called from ply_device_manager_free (), in this
+         * case we don't deactivate / close for retain-splash purposes.
+         */
+        if (close) {
+                if (manager->renderers_activated)
+                        ply_renderer_deactivate (renderer);
+
+                ply_renderer_close (renderer);
+        }
+
         ply_renderer_free (renderer);
 }
 
@@ -254,24 +301,18 @@ create_devices_for_udev_device (ply_device_manager_t *manager,
                                                                                  device_path,
                                                                                  terminal,
                                                                                  renderer_type);
+                        if (created) {
+                                if (renderer_type == PLY_RENDERER_TYPE_DRM)
+                                        manager->found_drm_device = 1;
+                                if (renderer_type == PLY_RENDERER_TYPE_FRAME_BUFFER)
+                                        manager->found_fb_device = 1;
+                        }
                 }
         }
 
         return created;
 }
 
-static void
-free_devices_for_udev_device (ply_device_manager_t *manager,
-                              struct udev_device   *device)
-{
-        const char *device_path;
-
-        device_path = udev_device_get_devnode (device);
-
-        if (device_path != NULL)
-                free_devices_from_device_path (manager, device_path);
-}
-
 static bool
 create_devices_for_subsystem (ply_device_manager_t *manager,
                               const char           *subsystem)
@@ -335,40 +376,149 @@ create_devices_for_subsystem (ply_device_manager_t *manager,
 }
 
 static void
-on_udev_event (ply_device_manager_t *manager)
+on_drm_udev_add_or_change (ply_device_manager_t *manager,
+                           const char           *action,
+                           const char           *device_path,
+                           struct udev_device   *device)
 {
-        struct udev_device *device;
-        const char *action;
+        ply_renderer_t *renderer;
+        bool changed;
 
-        device = udev_monitor_receive_device (manager->udev_monitor);
-        if (device == NULL)
+        renderer = ply_hashtable_lookup (manager->renderers, (void *) device_path);
+        if (renderer == NULL) {
+                /* We also try to create the renderer again on change events,
+                 * renderer creation fails when no outputs are connected and
+                 * this may have changed.
+                 */
+                create_devices_for_udev_device (manager, device);
                 return;
-
-        action = udev_device_get_action (device);
-
-        ply_trace ("got %s event for device %s", action, udev_device_get_sysname (device));
-
-        if (action == NULL)
-                return;
-
-        if (strcmp (action, "add") == 0) {
-                const char *subsystem;
-
-                subsystem = udev_device_get_subsystem (device);
-
-                if (strcmp (subsystem, SUBSYSTEM_DRM) == 0) {
-                        if (manager->local_console_managed && manager->local_console_is_text)
-                                ply_trace ("ignoring since we're already using text splash for local console");
-                        else
-                                create_devices_for_udev_device (manager, device);
-                } else {
-                        ply_trace ("ignoring since we only handle subsystem %s devices after timeout", subsystem);
-                }
-        } else if (strcmp (action, "remove") == 0) {
-                free_devices_for_udev_device (manager, device);
         }
 
-        udev_device_unref (device);
+        /* Renderer exists, bail if this is not a change event */
+        if (strcmp (action, "change"))
+                return;
+
+        changed = ply_renderer_handle_change_event (renderer);
+        if (changed) {
+                free_displays_for_renderer (manager, renderer);
+                create_pixel_displays_for_renderer (manager, renderer);
+        }
+}
+
+static bool
+verify_add_or_change (ply_device_manager_t *manager,
+                      const char           *action,
+                      const char           *device_path,
+                      struct udev_device   *device)
+{
+        const char *subsystem = udev_device_get_subsystem (device);
+
+        if (strcmp (action, "add") && strcmp (action, "change"))
+                return false;
+
+        subsystem = udev_device_get_subsystem (device);
+
+        if (strcmp (subsystem, SUBSYSTEM_DRM) == 0) {
+                if (manager->local_console_managed && manager->local_console_is_text) {
+                        ply_trace ("ignoring since we're already using text splash for local console");
+                        return false;
+                }
+        } else {
+                ply_trace ("ignoring since we only handle subsystem %s devices after timeout", subsystem);
+                return false;
+        }
+
+        return true;
+}
+
+static bool
+duplicate_device_path (ply_list_t *events, const char *device_path)
+{
+        struct udev_device *device;
+        ply_list_node_t *node;
+
+        for (node = ply_list_get_first_node (events);
+             node; node = ply_list_get_next_node (events, node)) {
+                device = ply_list_node_get_data (node);
+
+                if (strcmp (udev_device_get_devnode (device), device_path) == 0)
+                        return true;
+        }
+
+        return false;
+}
+
+static void
+process_udev_add_or_change_events (ply_device_manager_t *manager, ply_list_t *events)
+{
+        const char *action, *device_path;
+        struct udev_device *device;
+        ply_list_node_t *node;
+
+        while ((node = ply_list_get_first_node (events))) {
+                device = ply_list_node_get_data (node);
+                action = udev_device_get_action (device);
+                device_path = udev_device_get_devnode (device);
+
+                on_drm_udev_add_or_change (manager, action, device_path, device);
+
+                ply_list_remove_node (events, node);
+                udev_device_unref (device);
+        }
+}
+
+static void
+on_udev_event (ply_device_manager_t *manager)
+{
+        const char *action, *device_path;
+        struct udev_device *device;
+        ply_list_t *pending_events;
+
+        pending_events = ply_list_new ();
+
+        /*
+         * During the initial monitor/connector enumeration on boot the kernel
+         * fires a large number of change events. If we process these 1 by 1,
+         * we spend a lot of time probing the drm-connectors. So instead we
+         * collect them all and then coalescence them so that if there are multiple
+         * change events pending for a single card, we only re-probe the card once.
+         */
+        while ((device = udev_monitor_receive_device (manager->udev_monitor))) {
+                action = udev_device_get_action (device);
+                device_path = udev_device_get_devnode (device);
+
+                if (action == NULL || device_path == NULL)
+                        goto unref;
+
+                ply_trace ("got %s event for device %s", action, device_path);
+
+                /*
+                 * Add/change events before and after a remove may not be
+                 * coalesced together. So flush the queue and then process
+                 * the remove event immediately.
+                 */
+                if (strcmp (action, "remove") == 0) {
+                        process_udev_add_or_change_events (manager, pending_events);
+                        free_devices_from_device_path (manager, device_path, true);
+                        goto unref;
+                }
+
+                if (!verify_add_or_change (manager, action, device_path, device))
+                        goto unref;
+
+                if (duplicate_device_path (pending_events, device_path)) {
+                        ply_trace ("ignoring duplicate %s event for device %s", action, device_path);
+                        goto unref;
+                }
+
+                ply_list_append_data (pending_events, udev_device_ref(device));
+unref:
+                udev_device_unref (device);
+        }
+
+        process_udev_add_or_change_events (manager, pending_events);
+
+        ply_list_free (pending_events);
 }
 
 static void
@@ -377,7 +527,6 @@ watch_for_udev_events (ply_device_manager_t *manager)
         int fd;
 
         assert (manager != NULL);
-        assert (manager->udev_monitor == NULL);
 
         if (manager->fd_watch != NULL)
                 return;
@@ -476,7 +625,7 @@ free_renderer (char                 *device_path,
                ply_renderer_t       *renderer,
                ply_device_manager_t *manager)
 {
-        free_devices_from_device_path (manager, device_path);
+        free_devices_from_device_path (manager, device_path, false);
 }
 
 static void
@@ -818,8 +967,6 @@ create_non_graphical_devices (ply_device_manager_t *manager)
 static void
 create_devices_from_udev (ply_device_manager_t *manager)
 {
-        bool found_drm_device, found_fb_device;
-
         manager->device_timeout_elapsed = true;
 
         if (manager->paused) {
@@ -829,10 +976,10 @@ create_devices_from_udev (ply_device_manager_t *manager)
 
         ply_trace ("Timeout elapsed, looking for devices from udev");
 
-        found_drm_device = create_devices_for_subsystem (manager, SUBSYSTEM_DRM);
-        found_fb_device = create_devices_for_subsystem (manager, SUBSYSTEM_FRAME_BUFFER);
+        create_devices_for_subsystem (manager, SUBSYSTEM_DRM);
+        create_devices_for_subsystem (manager, SUBSYSTEM_FRAME_BUFFER);
 
-        if (found_drm_device || found_fb_device)
+        if (manager->found_drm_device || manager->found_fb_device)
                 return;
 
         ply_trace ("Creating non-graphical devices, since there's no suitable graphics hardware");
