@@ -139,6 +139,7 @@ struct _ply_boot_splash_plugin
         ply_image_t                        *header_image;
         ply_image_t                        *background_tile_image;
         ply_image_t                        *background_bgrt_image;
+        ply_image_t                        *background_bgrt_fallback_image;
         ply_image_t                        *watermark_image;
         ply_list_t                         *views;
 
@@ -173,6 +174,7 @@ struct _ply_boot_splash_plugin
         uint32_t                            progress_bar_bg_color;
         uint32_t                            progress_bar_fg_color;
 
+        double                              show_animation_fraction;
         progress_function_t                 progress_function;
 
         ply_trigger_t                      *idle_trigger;
@@ -470,7 +472,7 @@ view_set_bgrt_background (view_t *view)
                 panel_rotation = PLY_PIXEL_BUFFER_ROTATE_UPRIGHT;
         }
 
-        if (have_panel_props) {        
+        if (have_panel_props) {
                 ply_pixel_buffer_set_device_rotation (bgrt_buffer, panel_rotation);
                 ply_pixel_buffer_set_device_scale (bgrt_buffer, panel_scale);
         }
@@ -560,6 +562,30 @@ view_set_bgrt_background (view_t *view)
         }
 }
 
+static void
+view_set_bgrt_fallback_background (view_t *view)
+{
+        int width, height, x_offset, y_offset;
+        int screen_width, screen_height, screen_scale;
+        ply_pixel_buffer_t *image_buffer;
+
+        screen_width = ply_pixel_display_get_width (view->display);
+        screen_height = ply_pixel_display_get_height (view->display);
+        screen_scale = ply_pixel_display_get_device_scale (view->display);
+
+        image_buffer = ply_image_get_buffer (view->plugin->background_bgrt_fallback_image);
+
+        width = ply_pixel_buffer_get_width (image_buffer);
+        height = ply_pixel_buffer_get_height (image_buffer);
+        x_offset = (screen_width - width) / 2;
+        y_offset = screen_height * 382 / 1000 - height / 2;
+
+        view->background_buffer = ply_pixel_buffer_new (screen_width * screen_scale, screen_height * screen_scale);
+        ply_pixel_buffer_set_device_scale (view->background_buffer, screen_scale);
+        ply_pixel_buffer_fill_with_hex_color (view->background_buffer, NULL, 0x000000);
+        ply_pixel_buffer_fill_with_buffer (view->background_buffer, image_buffer, x_offset, y_offset);
+}
+
 static bool
 view_load (view_t *view)
 {
@@ -580,6 +606,9 @@ view_load (view_t *view)
 
         view_set_bgrt_background (view);
 
+        if (!view->background_buffer && plugin->background_bgrt_fallback_image != NULL)
+                view_set_bgrt_fallback_background (view);
+
         if (!view->background_buffer && plugin->background_tile_image != NULL) {
                 ply_trace ("tiling background to %lux%lu", screen_width, screen_height);
 
@@ -666,7 +695,7 @@ view_load (view_t *view)
                 ply_trace ("using %ldx%ld title centered at %ldx%ld for %ldx%ld screen",
                            width, title_height, x, y, screen_width, screen_height);
                 ply_label_show (view->title_label, view->display, x, y);
-                /* Use subtitle_height pixels seperation between title and subtitle */
+                /* Use subtitle_height pixels separation between title and subtitle */
                 y += title_height + subtitle_height;
         }
 
@@ -842,8 +871,8 @@ view_start_progress_animation (view_t *view)
                 view->animation_bottom = y + height;
         }
 
-        /* We don't really know how long shutdown will so
-         * don't show the progress animation
+        /* We don't really know how long shutdown will take,
+         * so don't show the progress animation
          */
         if (plugin->mode == PLY_BOOT_SPLASH_MODE_SHUTDOWN ||
             plugin->mode == PLY_BOOT_SPLASH_MODE_REBOOT)
@@ -905,7 +934,7 @@ view_show_prompt (view_t     *view,
 
                 view->lock_area.x =
                     view->dialog_area.x +
-                    (view->dialog_area.width - 
+                    (view->dialog_area.width -
                      (view->lock_area.width + entry_width)) / 2.0;
                 view->lock_area.y =
                     view->dialog_area.y +
@@ -1009,8 +1038,8 @@ load_mode_settings (ply_boot_splash_plugin_t *plugin,
         if (settings->use_firmware_background)
                 plugin->use_firmware_background = true;
 
-        settings->title = ply_key_file_get_value (key_file, group_name, "_Title");
-        settings->subtitle = ply_key_file_get_value (key_file, group_name, "_SubTitle");
+        settings->title = ply_key_file_get_value (key_file, group_name, "Title");
+        settings->subtitle = ply_key_file_get_value (key_file, group_name, "SubTitle");
 }
 
 static ply_boot_splash_plugin_t *
@@ -1020,6 +1049,7 @@ create_plugin (ply_key_file_t *key_file)
         char *image_dir, *image_path;
         char *transition;
         char *progress_function;
+        char *show_animation_fraction;
 
         srand ((int) ply_get_timestamp ());
         plugin = calloc (1, sizeof(ply_boot_splash_plugin_t));
@@ -1065,7 +1095,7 @@ create_plugin (ply_key_file_t *key_file)
                 ply_key_file_get_double (key_file, "two-step",
                                          "VerticalAlignment", 0.5);
 
-        /* Progressbar alignment, this defaults to the animation alignment
+        /* Progress bar alignment, this defaults to the animation alignment
          * for compatibility with older themes.
          */
         plugin->progress_bar_horizontal_alignment =
@@ -1150,9 +1180,14 @@ create_plugin (ply_key_file_t *key_file)
         load_mode_settings (plugin, key_file, "system-upgrade", PLY_BOOT_SPLASH_MODE_SYSTEM_UPGRADE);
         load_mode_settings (plugin, key_file, "firmware-upgrade", PLY_BOOT_SPLASH_MODE_FIRMWARE_UPGRADE);
 
-        if (plugin->use_firmware_background)
+        if (plugin->use_firmware_background) {
                 plugin->background_bgrt_image = ply_image_new ("/sys/firmware/acpi/bgrt/image");
 
+                asprintf (&image_path, "%s/bgrt-fallback.png", image_dir);
+                plugin->background_bgrt_fallback_image = ply_image_new (image_path);
+                free (image_path);
+        }
+
         plugin->dialog_clears_firmware_background =
                 ply_key_file_get_bool (key_file, "two-step", "DialogClearsFirmwareBackground");
 
@@ -1176,6 +1211,13 @@ create_plugin (ply_key_file_t *key_file)
                 free (progress_function);
         }
 
+        show_animation_fraction = ply_key_file_get_value (key_file, "two-step", "ShowAnimationPercent");
+        if (show_animation_fraction != NULL)
+                plugin->show_animation_fraction = strtod (show_animation_fraction, NULL);
+        else
+                plugin->show_animation_fraction = SHOW_ANIMATION_FRACTION;
+        free (show_animation_fraction);
+
         plugin->views = ply_list_new ();
 
         return plugin;
@@ -1243,6 +1285,9 @@ destroy_plugin (ply_boot_splash_plugin_t *plugin)
         if (plugin->background_bgrt_image != NULL)
                 ply_image_free (plugin->background_bgrt_image);
 
+        if (plugin->background_bgrt_fallback_image != NULL)
+                ply_image_free (plugin->background_bgrt_fallback_image);
+
         if (plugin->watermark_image != NULL)
                 ply_image_free (plugin->watermark_image);
 
@@ -1336,7 +1381,7 @@ start_progress_animation (ply_boot_splash_plugin_t *plugin)
 
         plugin->is_animating = true;
 
-        /* We don't really know how long shutdown will, take
+        /* We don't really know how long shutdown will take,
          * but it's normally really fast, so just jump to
          * the end animation
          */
@@ -1394,9 +1439,12 @@ draw_background (view_t             *view,
         ply_boot_splash_plugin_t *plugin;
         ply_rectangle_t area;
         bool use_black_background = false;
+        bool using_fw_background;
 
         plugin = view->plugin;
 
+        using_fw_background = (plugin->background_bgrt_image || plugin->background_bgrt_fallback_image);
+
         area.x = x;
         area.y = y;
         area.width = width;
@@ -1405,7 +1453,7 @@ draw_background (view_t             *view,
         /* When using the firmware logo as background and we should not use
          * it for this mode, use solid black as background.
          */
-        if (plugin->background_bgrt_image &&
+        if (using_fw_background &&
             !plugin->mode_settings[plugin->mode].use_firmware_background)
                 use_black_background = true;
 
@@ -1414,7 +1462,7 @@ draw_background (view_t             *view,
          */
         if ((plugin->state == PLY_BOOT_SPLASH_DISPLAY_QUESTION_ENTRY ||
              plugin->state == PLY_BOOT_SPLASH_DISPLAY_PASSWORD_ENTRY) &&
-            plugin->background_bgrt_image && plugin->dialog_clears_firmware_background)
+            using_fw_background && plugin->dialog_clears_firmware_background)
                 use_black_background = true;
 
         if (use_black_background)
@@ -1603,8 +1651,6 @@ show_splash_screen (ply_boot_splash_plugin_t *plugin,
                     ply_buffer_t             *boot_buffer,
                     ply_boot_splash_mode_t    mode)
 {
-        int i;
-
         assert (plugin != NULL);
 
         plugin->loop = loop;
@@ -1657,9 +1703,14 @@ show_splash_screen (ply_boot_splash_plugin_t *plugin,
                 } else {
                         ply_image_free (plugin->background_bgrt_image);
                         plugin->background_bgrt_image = NULL;
-                        for (i = 0; i < PLY_BOOT_SPLASH_MODE_COUNT; i++)
-                                plugin->mode_settings[i].use_firmware_background = false;
-                        plugin->use_firmware_background = false;
+                }
+        }
+
+        if (plugin->background_bgrt_fallback_image != NULL) {
+                ply_trace ("loading background bgrt fallback image");
+                if (!ply_image_load (plugin->background_bgrt_fallback_image)) {
+                        ply_image_free (plugin->background_bgrt_fallback_image);
+                        plugin->background_bgrt_fallback_image = NULL;
                 }
         }
 
@@ -1753,7 +1804,7 @@ on_boot_progress (ply_boot_splash_plugin_t *plugin,
          * become_idle gets called.
          */
         if (plugin->mode_settings[plugin->mode].use_end_animation &&
-            fraction_done >= SHOW_ANIMATION_FRACTION) {
+            fraction_done >= plugin->show_animation_fraction) {
                 if (plugin->stop_trigger == NULL) {
                         ply_trace ("boot progressed to end");
 
@@ -1767,7 +1818,7 @@ on_boot_progress (ply_boot_splash_plugin_t *plugin,
         } else {
                 double total_duration;
 
-                fraction_done *= (1 / SHOW_ANIMATION_FRACTION);
+                fraction_done *= (1 / plugin->show_animation_fraction);
 
                 switch (plugin->progress_function) {
                 /* Fun made-up smoothing function to make the growth asymptotic:
